;----------------------------------------------
; Here's a routine for creating binary coded decimal
; byte sequences from unsigned integer input!
; Mathew Carr - 2012/03/18
; The BCD sequence is a series of bytes containing the
; values of the decimal places, in order from
; most significant digit (earliest address) to least
; significant digit (latest address)

BCDBS_BYTE_SEQUENCE_LENGTH = 10

; Analyse the input value and put the digits in the
; pointed to series.
; Args:
;   d0 = value
;   a0 = pointer to byte sequence (lowest element)
; Output:
;   d7 = number of digits in the sequence. (An result of '0' counts as one digit)
BCDbs_CreateSequence:
        movem.l BCDbs_CreateSequence_regs,-(a7)

        ; Clear the sequence first
        move.l  #BCDBS_BYTE_SEQUENCE_LENGTH-1,d1
        move.l  a0,a1
BCDbs_CreateSequence_initialclearloop:
        move.b  #0,(a1)+
        dbf     d1,BCDbs_CreateSequence_initialclearloop
        ; The sequence clearing has completed.

        ; Special case, check for zero.
        ; If zero, then no more needs to be done: the sequence is a series of zeroes and the length is 1.
        tst.l   d0
        beq     BCDbs_CreateSequence_valueiszero

        ; Skip leading zeroes. (Don't double the BCD sequence until we hit the first 1.)
        move.l  #32-1,d1                                                         ; d1 holds number of input bits to consider. (minus 1 for dbf)
BCDbs_CreateSequence_shiftnextinputbit_leadingzerosstate:
        rol.l   d0                                                               ; Put the MSB of d0 into LSB for anding.
        move.b  d0,d2
        and.b   #1,d2                                                            ; d2 = MSB of d0
        bne     BCDbs_CreateSequence_shiftnextinputbit_receivedfirstnonzerobit   ; When we find the first non-zero, then start the doubling state.
        dbf     d1,BCDbs_CreateSequence_shiftnextinputbit_leadingzerosstate      ; The only exit of this loop is finding the set bit (because we know d0 isn't zero)

BCDbs_CreateSequence_shiftnextinputbit:
        rol.l   d0                                                               ; Put the MSB of d0 into LSB for anding.
        move.b  d0,d2
        and.b   #1,d2                                                            ; d2 = MSB of d0
BCDbs_CreateSequence_shiftnextinputbit_receivedfirstnonzerobit:
        ; Okay, now the carry into bit is d2, and the cell under consideration is at a1-1
        move.l  a1,a2                                                            ; a2 will be our cursor.
        move.l  #BCDBS_BYTE_SEQUENCE_LENGTH-1,d3                                 ; d3 is the number of bytes left to consider. (-1 for dbf)
BCDbs_CreateSequence_propagatetonextbyte:
        move.b  -(a2),d4                                                         ; d4 is the original value in the cell
        move.b  #0,d5                                                            ; d5 is the carry bit for the NEXT cell
        cmp.b   #5,d4                                                            ; Is d4 < 5
        blt     BCDbs_CreateSequence_dontsetcarry                                ; If it is, it doesn't set the carry for the next cell.
        sub.b   #5,d4                                                            ; If d4 >= 5, we subtract five and set the carry bit for the next cell.
        move.b  #1,d5                                                            ; d5 is the carry bit for the next cell
BCDbs_CreateSequence_dontsetcarry:
        asl.b   #1,d4                                                            ; Double d4 to get new value.
        or.b    d2,d4                                                            ; If the carry bit is set, we +1 to our new value.
        move.b  d4,(a2)                                                          ; Write the final value into the cell
        move.b  d5,d2                                                            ; Put the d5 set carry bit value into the d2 current cell variable (for the next cell)
        ; We want to propagate these changes across all bytes in the BCD sequence.
        dbf     d3,BCDbs_CreateSequence_propagatetonextbyte
        ; And we want to do this for all the bits in the input
        dbf     d1,BCDbs_CreateSequence_shiftnextinputbit
        ; DONE.
        ; Now do the byte sequence length check.
        move.l  #BCDBS_BYTE_SEQUENCE_LENGTH-1,d7                                 ; Check all the cells in the output sequence (-1 for dbf)
BCDbs_CreateSequence_digitcalc_checknextbyte:
        cmp.b   #0,(a0)+
        bne     BCDbs_CreateSequence_digitcalc_donechecking                      ; Exit loop when a nonzero cell appears (return that in d7!)
        dbf     d7,BCDbs_CreateSequence_digitcalc_checknextbyte                  ; If the cell is nonzero, subtract one from d7
BCDbs_CreateSequence_digitcalc_donechecking:
        add.l   #1,d7                                                            ; We have to addone because dbf has loop counter set one less than proper value.
        movem.l (a7)+,BCDbs_CreateSequence_regs
        rts
BCDbs_CreateSequence_valueiszero:
        move.l  #1,d7
        movem.l (a7)+,BCDbs_CreateSequence_regs
        rts
BCDbs_CreateSequence_regs reg d0-d6/a0-a2